==> phylodm-2.1.2.tar.gz: phylodm-2.1.2/src/python.rs <==
use numpy::{PyArray2, ToPyArray};
use pyo3::prelude::*;
use pyo3::{pymodule, types::PyModule, PyResult, Python};

use crate::pdm::PDM as Tree;
use crate::tree::{Edge, NodeId, Taxon};

#[pyclass]
struct PhyloDM {
    tree: Tree,
}

#[pymethods]
impl PhyloDM {
    #[new]
    fn new() -> Self {
        Self {
            tree: Tree::default(),
        }
    }

    pub fn load_from_newick_path(&mut self, path: &str) {
        self.tree.load_from_newick_path(path);
    }

    pub fn add_node(&mut self, taxon: Option<String>) -> usize {
        match taxon {
            Some(taxon) => self.tree.add_node(Some(Taxon(taxon))),
            None => self.tree.add_node(None),
        }
        .0
    }

    pub fn add_edge(&mut self, parent_id: usize, child_id: usize, length: f64) {
        self.tree
            .add_edge(NodeId(parent_id), NodeId(child_id), Edge(length));
    }

    pub fn get_nodes(&self) -> Vec<usize> {
        let mut out = Vec::new();
        for node in &self.tree.nodes {
            out.push(node.id.0);
        }
        out
    }

    pub fn dm(&mut self, norm: bool) -> Py<PyArray2<f64>> {
        let (_, array) = self.tree.matrix(norm);
        Python::with_gil(|py| {
            return Py::from(array.to_pyarray(py));
        })
    }

    pub fn taxa(&self) -> Vec<String> {
        let mut out: Vec<String> = Vec::new();
        for taxon in self.tree.leaf_nodes() {
            out.push(taxon.0);
        }
        out
    }

    pub fn length(&self) -> f64 {
        self.tree.length().0
    }
}

#[pymodule]
fn pdm(_py: Python<'_>, m: &PyModule) -> PyResult<()> {
    m.add_class::<PhyloDM>()?;
    Ok(())
}

==> phylodm-2.2.0.tar.gz: phylodm-2.2.0/src/python.rs <==
use numpy::{PyArray2, ToPyArray};
use pyo3::{Py, pyclass, pymethods, pymodule, PyResult, Python, types::PyModule};

use crate::pdm::PDM as RustPhyloDM;
use crate::tree::{Edge, NodeId, Taxon};

#[pyclass]
struct PhyloDM {
    tree: RustPhyloDM,
}

#[pymethods]
impl PhyloDM {
    #[new]
    fn new() -> Self {
        Self {
            tree: RustPhyloDM::default(),
        }
    }

    pub fn load_from_newick_path(&mut self, path: &str) {
        self.tree.load_from_newick_path(path);
    }

    pub fn add_node(&mut self, taxon: Option<&str>) -> usize {
        match taxon {
            Some(taxon) => self.tree.add_node(Some(&Taxon(taxon.to_string()))),
            None => self.tree.add_node(None),
        }
            .0
    }

    pub fn add_edge(&mut self, parent_id: usize, child_id: usize, length: f64) {
        self.tree.add_edge(
            NodeId(parent_id),
            NodeId(child_id),
            Edge(length),
        );
    }

    pub fn get_nodes(&self) -> Vec<usize> {
        let mut out = Vec::new();
        for node in &self.tree.nodes {
            out.push(node.id.0);
        }
        out
    }

    pub fn dm(&mut self, norm: bool) -> Py<PyArray2<f64>> {
        let (_, array) = self.tree.matrix(norm);
        Python::with_gil(|py| {
            return Py::from(array.to_pyarray(py));
        })
    }

    pub fn taxa(&self) -> Vec<String> {
        let mut out: Vec<String> = Vec::new();
        for taxon in self.tree.leaf_nodes() {
            out.push(taxon.0);
        }
        out
    }

    pub fn length(&self) -> f64 {
        self.tree.length().0
    }
    
    pub fn compute_row_vec(&mut self) {
        self.tree.compute_row_vec();
    }
    
    pub fn distance(&self, a: &str, b: &str, norm: bool) -> f64 {
        let taxon_a = Taxon(a.to_string());
        let taxon_b = Taxon(b.to_string());
        self.tree.distance(&taxon_a, &taxon_b, norm)
    }
}

#[pymodule]
fn pdm(_py: Python<'_>, m: &PyModule) -> PyResult<()> {
    m.add_class::<PhyloDM>()?;
    Ok(())
}
